<?php

namespace FiloBlu\Flow\Model\From;

class Multistock extends AbstractFrom {

    /**
     * @var Monolog\Logger
     */
    protected $_logger;
    protected $_helper_flow;
    protected $_product_factory;
    protected $_stock_registry;

    protected $fullReindexAction;
    protected $processor;
    protected $stockRepository;
    protected $warehouseRepositoryInterface;

    /**
     * @param \Magento\Framework\Model\Context $context
     * @param \Magento\Framework\Registry $registry
     * @param \Magento\Catalog\Model\ProductFactory $productFactory
     * @param \FiloBlu\Flow\Helper\LoggerProvider $loggerProvider
     * @param array $data
     */
    public function __construct(
        \Magento\Framework\Model\Context $context,
        \Magento\Framework\Registry $registry,
        \FiloBlu\Flow\Helper\Data $helper_flow,
        \FiloBlu\Flow\Helper\LoggerProvider $loggerProvider,
        \Magento\Catalog\Model\ProductFactory $productFactory,
        \Magento\Framework\App\ProductMetadataInterface $_productMetadata,

        \FiloBlu\Flow\Model\AmastyMultiInventory\FullFactory $fullAmastyFactoryCreator,
        //\Amasty\MultiInventory\Model\Indexer\Warehouse\Action\Full $fullReindexAction,
        //\Amasty\MultiInventory\Model\Indexer\Warehouse\Processor $processor,
        //\Amasty\MultiInventory\Api\WarehouseItemRepositoryInterface $stockRepository,
        //\Amasty\MultiInventory\Api\WarehouseRepositoryInterface $warehouseRepositoryInterface,

        array $data = []
    ) {
        $this->_helper_flow = $helper_flow;
        $this->_logger = $loggerProvider->getLogger();
        $this->_product_factory = $productFactory;
        $this->_productMetadata = $_productMetadata;

        $this->fullReindexAction = $fullAmastyFactoryCreator->create($data,"\Amasty\MultiInventory\Model\Indexer\Warehouse\Action\Full");
        $this->processor = $fullAmastyFactoryCreator->create($data,"\Amasty\MultiInventory\Model\Indexer\Warehouse\Processor");
        $this->stockRepository = $fullAmastyFactoryCreator->create($data,"\Amasty\MultiInventory\Api\WarehouseItemRepositoryInterface");
        $this->warehouseRepositoryInterface = $fullAmastyFactoryCreator->create($data,"\Amasty\MultiInventory\Api\WarehouseRepositoryInterface");

        parent::__construct($context, $registry, null, null, $data);
    }

    /**
     * @return void
     */
    public function _construct()
    {
        $this->_init('FiloBlu\Flow\Model\ResourceModel\From\Stock');
    }

    public function process(){

        $product_id = $this->_product_factory->create()->getIdBySku($this->getSku());

        if(!$product_id){
            $message = sprintf("Product not found! product_code: '%s'", $this->getSku());
            return $this->error(false, $message);
        }
        $warehouse_id = $this->getWarehouseIdFromCode($this->getMagazzino());
        if(!$warehouse_id){
            $message = sprintf("Warehouse {$this->getMagazzino()} not found! product_code: '%s'", $this->getSku());
            return $this->error(false, $message);
        }
        if (!$this->fullReindexAction || !$this->processor || !$this->stockRepository || !$this->warehouseRepositoryInterface){
            $message = sprintf("Module Amasty_MultiInventory not found!");
            return $this->error(false, $message);
        }

        if($product_id) {

            try {
                // salvo il prezzo sul prodotto semplice tramite amasty
                $item = $this->stockRepository->getByProductWarehouse($product_id, $warehouse_id);
                $item->setProductId($product_id);
                $item->setWarehouseId($warehouse_id);
                $item->setQty($this->getData('fisico_disponibile'));
                $item->recalcAvailable();
                $this->stockRepository->save($item);

                // Moved on postprocessFileRows
                //$this->fullReindexAction->setUpdateStockStatus(true);
                //$this->processor->reindexAll();

                $this->_logger->info('Process STOCK - qty ' . $this->getData('fisico_disponibile') . ' ID: ' . $product_id);
            } catch (\Exception $e) {
                $this->_logger->info('Errore nel salvataggio dello stock. ' . $e->getMessage());
                exit($e->getMessage());
            }

        }

        return $this;
    }

    protected function postprocessFileRows($query) {

        try {
            $this->fullReindexAction->setUpdateStockStatus(true);
            $this->processor->reindexAll();
        } catch(\Exception $e){
            $this->_logger->info('Errore nel reindex Amasty post import multistock. ' . $e->getMessage());
            exit($e->getMessage());
            return false;
        }

        return true;

    }

    protected function getWarehouseIdFromCode($code){

        try {
            $warehouse = $this->warehouseRepositoryInterface->getByCode($code);
            $id = $warehouse->getId();
        } catch (\Exception $ex) {
            $id = null;
        }

        return $id;
    }


}